#include "gpio.hpp"

#include <stdlib.h>
#include "driver/gpio.h"

extern "C" {
    void config_output_gpio(int pin){
        gpio_num_t pin2 = static_cast<gpio_num_t>(pin);
        gpio_reset_pin(pin2);
        gpio_intr_disable(pin2);
        gpio_set_direction(pin2, GPIO_MODE_OUTPUT);
        gpio_pulldown_dis(pin2);
        gpio_pullup_dis(pin2);
    }
    void config_input_gpio(int pin){
        gpio_num_t pin2 = static_cast<gpio_num_t>(pin);
        gpio_reset_pin(pin2);
        gpio_intr_disable(pin2);
        gpio_set_direction(pin2, GPIO_MODE_INPUT);
        gpio_pulldown_dis(pin2);
        gpio_pullup_dis(pin2);
    }
    bool read_gpio_pin(int pin){
        gpio_num_t pin2 = static_cast<gpio_num_t>(pin);
        return gpio_get_level(pin2) ? true : false;
    }
    void write_gpio_pin(int pin, bool value){
        gpio_num_t pin2 = static_cast<gpio_num_t>(pin);
        gpio_set_level(pin2, value?1:0);
    }
}

GPIOPin::GPIOPin(int pin, bool compliment){ this->pin = pin; this->compliment = compliment; }
int GPIOPin::get_pin(){ return this->pin; }
void GPIOPin::set_bound(){ this->bound = true; }
void GPIOPin::ensure_bound(){ if (!this->bound) this->bind(); }
bool GPIOPin::apply_compliment_if_enabled(bool value){ return (this->compliment) ? (value ? 0 : 1) : (value ? 1 : 0); }

void GPIOInputPin::bind(){ config_input_gpio(this->get_pin()); this->set_bound(); }
void GPIOOutputPin::bind(){ config_output_gpio(this->get_pin()); this->set_bound(); this->write(false); }

bool GPIOInputPin::read(){
    this->ensure_bound();
    return this->apply_compliment_if_enabled(read_gpio_pin(this->get_pin()));
}
void GPIOOutputPin::write(bool value){
    this->ensure_bound();
    bool complimented = this->apply_compliment_if_enabled(value);
    write_gpio_pin(this->get_pin(), complimented);
    this->last_written = value;
}
void GPIOOutputPin::flip(){
    this->ensure_bound();
    this->write(!this->last_written);
}
bool GPIOOutputPin::get_last_written(){return this->last_written;}
