#include "motorlogic.hpp"
#include "../utils/swap.hpp"

MotorLogic::MotorLogic(VFS* vfs, MotorControl* mc){
    this->vfs = vfs;
    this->mutex = new Mutex();
    this->pending_actions = new std::queue<Action*>();
    this->motor_control = mc;
    mc->motor_logic = this;
};

void MotorLogic::mainloop(){
    int rhythm = this->vfs->nvs->get_int("default_rhyrhm", 90);
    if(rhythm<=0){
        rhythm = 1;
    }
    int microsleep = 0;
    int min_l_pos = 0;
    int min_r_pos = 0;
    int max_l_pos = 0;
    int max_r_pos = 0;
    int dlt_l_pos = 1;
    int dlt_r_pos = 1;
    std::function<void()> update_params = [&](){
        microsleep = this->vfs->nvs->get_int("microsleep", 0);
        if(microsleep<=0){
            microsleep = 400;
        }
        min_l_pos = this->vfs->nvs->get_int("min_l_pos", 0);
        min_r_pos = this->vfs->nvs->get_int("min_r_pos", 0);
        max_l_pos = this->vfs->nvs->get_int("max_l_pos", 0);
        max_r_pos = this->vfs->nvs->get_int("max_r_pos", 0);
        if(max_l_pos<min_l_pos)
            swap_ref<int>(max_l_pos, min_l_pos);
        else if(min_l_pos==max_l_pos)
            max_l_pos+=1;
        if(max_r_pos<min_r_pos)
            swap_ref<int>(max_r_pos, min_r_pos);
        else if(min_r_pos==max_r_pos)
            max_r_pos+=1;
        dlt_l_pos = max_l_pos - min_l_pos;
        dlt_r_pos = max_r_pos - min_r_pos;
    };
    update_params();
    Coreographer* coreographer = new CoreographerHoldingPosition(0, 0);
    while (true) {
        this->mutex->run([&](){
            if(this->pending_actions->size()){
                auto* action = this->pending_actions->front();
                action->do_action(this->motor_control, coreographer, rhythm, update_params);
                this->pending_actions->pop();
                delete action;
            }
        });
        coreographer->set_boundaries(min_l_pos, max_l_pos, min_r_pos, max_r_pos);
        coreographer->setup_stepper(this->motor_control->left_stepper, this->motor_control->right_stepper, rhythm);
        this->motor_control->left_stepper->pursue_target();
        this->motor_control->right_stepper->pursue_target();
        if(this->motor_control->left_stepper->is_target_reached() && this->motor_control->right_stepper->is_target_reached()){
            this->motor_control->onboard_led->write(false);
        }else{
            this->motor_control->onboard_led->write(true);
        }
        usleep(microsleep);
    }
}
