#include "steppermotor.hpp"

StepperMotor::StepperMotor(StepperMotorPins* sp){
    this->stepper_actuators = new StepperMotorActuators(sp);
    this->disable_steppers();
}

// Target getter/setter
void StepperMotor::set_target(int tgt){this->target = tgt;}
int StepperMotor::get_target(){return this->target;}
int StepperMotor::get_current(){return this->current;}

// Target GPIO action
bool StepperMotor::is_target_reached(){ return this->current == this->target; }
void StepperMotor::pursue_target(){
    int diff = this->target - this->current;
    if(diff>0) this->rotate(diff);
    else if(diff<0) this->rotate_inverse(-diff);
}

int StepperMotor::plan_rotation_size(int aimed_step_size){
    auto* mp = this->stepper_actuators->stepper_motor_pins->driver_steppings;
    int max_step_size = mp->at(-1);
    if(aimed_step_size<=0 || aimed_step_size>=max_step_size)
        return max_step_size;
    int last_valid_step_size = 1;
    for(auto& pair: *mp){
        int key = pair.first;
        if(key <= 0) continue;
        if(key>last_valid_step_size && key<=aimed_step_size)
            last_valid_step_size = key;
    }
    return last_valid_step_size;
}

void StepperMotor::set_step_size(int step_size){
    auto* mp = this->stepper_actuators->stepper_motor_pins->driver_steppings;
    int precision_pins = mp->at(step_size);
    uint8_t sz = this->stepper_actuators->microstep->size();
    for(uint8_t i=0; i<sz; i++){
        GPIOOutputPin* gpio = this->stepper_actuators->microstep->at(i);
        int precision_pin = (precision_pins & (1 << i)) ? 1 : 0;
        gpio->write(precision_pin);
    }
}

// Blindly rotate (updates current)
// but that's just a detail [step_size<0 --> Full Step]
// @return number of units covered by step
int StepperMotor::rotate(int aimed_step_size, bool direction){
    if(aimed_step_size<0) return this->rotate(-aimed_step_size, !direction);
    if(this->stepper_actuators->step->get_last_written()){
        this->stepper_actuators->step->flip();
        return 0;
    }
    int step_size = this->plan_rotation_size(aimed_step_size);
    this->set_step_size(step_size);
    this->stepper_actuators->direction->write(direction);
    this->stepper_actuators->step->flip();
    this->current += direction ? -step_size : step_size;
    return step_size;
}
int StepperMotor::rotate_inverse(int aimed_step_size){
    return this->rotate(aimed_step_size, true);
}

// Forgets about both current state and target
void StepperMotor::reset_state(){
    this->current = 0;
    this->target = 0;
}

// Enables/disables steppers (might require recalibration)
void StepperMotor::enable_steppers(){
    this->stepper_actuators->enable->write(true);
    this->engaged=true;
}
void StepperMotor::disable_steppers(){
    this->stepper_actuators->enable->write(false);
    this->engaged=false;
    this->reset_state();
}
void StepperMotor::ensure_engaged(){
    if(!this->engaged) this->enable_steppers();
}
