#ifndef STEPPERMOTOR_HPP
#define STEPPERMOTOR_HPP
#include "steppermotorpins.hpp"
#include "steppermotoractuators.hpp"
#include <map>

class StepperMotor {
protected:
    StepperMotorActuators* stepper_actuators;
    int target = 0;
    int current = 0;
    bool engaged = false;
    int max_step_size;
    std::map<int, int>* steppings;
    void ensure_engaged();
    int plan_rotation_size(int aimed_step_size=-1);
    void set_step_size(int step_size);
public:
    StepperMotor(StepperMotorPins* sp);

    // Target getter/setter
    void set_target(int tgt);
    int get_target();
    int get_current();

    // Target GPIO action
    void pursue_target();
    bool is_target_reached();

    // Blindly rotate (updates current, but that's just a detail [step_size<0 --> Full Step])
    // @return number of units covered by step
    int rotate(int step_size = -1, bool direction = false);
    int rotate_inverse(int step_size = -1);

    // Forgets about both current state and target
    void reset_state();

    // Enables/disables steppers (might require recalibration)
    void enable_steppers(); // engaged=true
    void disable_steppers(); // engaged=false & call reset_state
};

#endif
