#include "nvs.hpp"

NVS::NVS(){
    this->namespc = "storage";
}

Array<char>* NVS::get_string(std::string location){
    esp_err_t err;
    err = nvs_open(this->namespc, NVS_READWRITE, &(this->handle));
    if (err != ESP_OK){
        nvs_close(this->handle);
        return nullptr;
    }
    size_t required_size = 0;
    err = nvs_get_blob(this->handle, location.c_str(), NULL, &required_size);
    if (err != ESP_OK){
        nvs_close(this->handle);
        return nullptr;
    }
    auto* data = new Array<char>(required_size);
    if(data->array == NULL){
        nvs_close(this->handle);
        return nullptr;
    }
    nvs_get_blob(this->handle, location.c_str(), data->array, &required_size);
    return data;
}

Array<uint8_t>* NVS::get_blob(std::string location){
    esp_err_t err;
    err = nvs_open(this->namespc, NVS_READONLY, &(this->handle));
    if (err != ESP_OK){
        nvs_close(this->handle);
        return nullptr;
    }
    size_t required_size = 0;
    err = nvs_get_blob(this->handle, location.c_str(), NULL, &required_size);
    if (err != ESP_OK){
        nvs_close(this->handle);
        return nullptr;
    }
    auto* data = new Array<uint8_t>(required_size);
    if(data->array == NULL){
        nvs_close(this->handle);
        return nullptr;
    }
    nvs_get_blob(this->handle, location.c_str(), data->array, &required_size);
    return data;
}

void NVS::set_string(std::string location, std::string text){
    nvs_open(this->namespc, NVS_READWRITE, &(this->handle));
    auto cstr = text.c_str();
    nvs_set_blob(this->handle, location.c_str(), cstr, strlen(cstr)+1);
    nvs_commit(this->handle);
    nvs_close(this->handle);
}

void NVS::set_blob(std::string location, Array<uint8_t>* blob){
    nvs_open(this->namespc, NVS_READWRITE, &(this->handle));
    nvs_set_blob(this->handle, location.c_str(), blob->array, blob->size);
    nvs_commit(this->handle);
    nvs_close(this->handle);
}

void NVS::rm(std::string location){
    nvs_open(this->namespc, NVS_READWRITE, &(this->handle));
    nvs_erase_key(this->handle, location.c_str());
    nvs_commit(this->handle);
    nvs_close(this->handle);
}

int NVS::get_int(std::string location, int failure){
    int result = failure;
    auto* str = this->get_string(location);
    if(str!=nullptr){
        result = atoi(str->array);
    }
    delete str;
    return result;
}

long NVS::get_long(std::string location, long failure){
    long result = failure;
    auto* str = this->get_string(location);
    if(str!=nullptr){
        result = atol(str->array);
    }
    delete str;
    return result;
}

double NVS::get_float(std::string location, double failure){
    double result = failure;
    auto* str = this->get_string(location);
    if(str!=nullptr){
        result = atof(str->array);
    }
    delete str;
    return result;
}

void NVS::set_int(std::string location, int value){
    String s("");
    s.concat(value);
    this->set_string(location, s.c_str());
}

void NVS::set_long(std::string location, long value){
    String s("");
    s.concat(value);
    this->set_string(location, s.c_str());
}

void NVS::set_float(std::string location, double value){
    String s("");
    s.concat(value);
    this->set_string(location, s.c_str());
}

